;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'erc-match '("erc-match"))
(custom-add-loads 'erc-log '("erc-log"))
(custom-add-loads 'erc-nicklist '("erc-nicklist"))
(custom-add-loads 'erc-pcomplete '("erc-pcomplete"))
(custom-add-loads 'erc-dcc '("erc-dcc" "erc-xdcc"))
(custom-add-loads 'erc-page '("erc-page"))
(custom-add-loads 'erc-scripts '("erc"))
(custom-add-loads 'erc '("erc-autoaway" "erc-autojoin" "erc-bbdb" "erc-button" "erc-chess" "erc-complete" "erc-dcc" "erc-ezbounce" "erc-fill" "erc-goodies" "erc-ibuffer" "erc-list" "erc-log" "erc-match" "erc-nets" "erc-netsplit" "erc-nicklist" "erc-nickserv" "erc-notify" "erc-page" "erc-pcomplete" "erc-replace" "erc-sound" "erc-speak" "erc-speedbar" "erc-stamp" "erc-track" "erc-truncate" "erc"))
(custom-add-loads 'erc-paranoia '("erc"))
(custom-add-loads 'erc-coding-systems '("erc"))
(custom-add-loads 'applications '("erc"))
(custom-add-loads 'erc-control-characters '("erc-goodies"))
(custom-add-loads 'erc-stamp '("erc-stamp"))
(custom-add-loads 'erc-ezbounce '("erc-ezbounce"))
(custom-add-loads 'erc-sound '("erc-sound"))
(custom-add-loads 'erc-server-hooks '("erc"))
(custom-add-loads 'erc-speak '("erc-speak"))
(custom-add-loads 'erc-hooks '("erc-members" "erc-netsplit" "erc"))
(custom-add-loads 'erc-autoaway '("erc-autoaway"))
(custom-add-loads 'erc-list '("erc-list"))
(custom-add-loads 'erc-buffers '("erc"))
(custom-add-loads 'erc-netsplit '("erc-netsplit"))
(custom-add-loads 'erc-services '("erc-nickserv"))
(custom-add-loads 'erc-autojoin '("erc-autojoin"))
(custom-add-loads 'erc-quit-and-part '("erc"))
(custom-add-loads 'erc-chess '("erc-chess"))
(custom-add-loads 'erc-networks '("erc-nets"))
(custom-add-loads 'erc-notify '("erc-notify"))
(custom-add-loads 'erc-display '("erc-goodies" "erc"))
(custom-add-loads 'erc-ibuffer '("erc-ibuffer"))
(custom-add-loads 'erc-mode-line-and-header '("erc"))
(custom-add-loads 'erc-replace '("erc-replace"))
(custom-add-loads 'erc-old-complete '("erc-complete"))
(custom-add-loads 'erc-fill '("erc-fill"))
(custom-add-loads 'erc-query '("erc"))
(custom-add-loads 'erc-ignore '("erc"))
(custom-add-loads 'erc-faces '("erc-button" "erc-goodies" "erc-list" "erc-match" "erc-stamp" "erc"))
(custom-add-loads 'erc-button '("erc-button"))
(custom-add-loads 'erc-speedbar '("erc-speedbar"))
(custom-add-loads 'erc-track '("erc-track"))
(custom-add-loads 'erc-bbdb '("erc-bbdb"))
(custom-add-loads 'erc-truncate '("erc-truncate"))

;;; custom-load.el ends here
